@@ -24,6 +24,8 @@ use tui::{
 };
 use webbrowser::Browser;
 
+const TICK_RATE_MS: u64 = 100;
+
 /// a sleek typing tui written in rust
 #[derive(Parser, Debug, Clone)]
 #[clap(version, about, long_about= None)]
@@ -73,9 +75,12 @@ impl App {
 
             language.get_random(cli.number_of_words).join(" ")
         };
-
         Self {
-            thok: Thok::new(prompt, cli.number_of_secs),
+            thok: Thok::new(
+                prompt,
+                cli.number_of_words,
+                cli.number_of_secs.map(|ns| ns as f64),
+            ),
             cli: Some(cli),
         }
     }
@@ -91,7 +96,11 @@ impl App {
             }
         };
 
-        self.thok = Thok::new(prompt, cli.number_of_secs);
+        self.thok = Thok::new(
+            prompt,
+            cli.number_of_words,
+            cli.number_of_secs.map(|ns| ns as f64),
+        );
     }
 }
 
@@ -137,7 +146,7 @@ fn start_tui<B: Backend>(
 
     loop {
         let mut exit_type: ExitType = ExitType::Quit;
-        terminal.draw(|f| ui(f, app))?;
+        terminal.draw(|f| ui(app, f))?;
 
         loop {
             let app = &mut app;
@@ -150,11 +159,11 @@ fn start_tui<B: Backend>(
                         if app.thok.has_finished() {
                             app.thok.calc_results();
                         }
-                        terminal.draw(|f| ui(f, app))?;
+                        terminal.draw(|f| ui(app, f))?;
                     }
                 }
                 ThokEvent::Resize => {
-                    terminal.draw(|f| ui(f, app))?;
+                    terminal.draw(|f| ui(app, f))?;
                 }
                 ThokEvent::Key(key) => {
                     match key.code {
@@ -201,7 +210,7 @@ fn start_tui<B: Backend>(
                         },
                         _ => {}
                     }
-                    terminal.draw(|f| ui(f, app))?;
+                    terminal.draw(|f| ui(app, f))?;
                 }
             }
         }
@@ -235,26 +244,29 @@ fn get_thok_events(should_tick: bool) -> mpsc::Receiver<ThokEvent> {
     if should_tick {
         let tick_x = tx.clone();
         thread::spawn(move || loop {
-            tick_x.send(ThokEvent::Tick).unwrap();
-            thread::sleep(Duration::from_millis(100))
+            if tick_x.send(ThokEvent::Tick).is_err() {
+                break;
+            }
+
+            thread::sleep(Duration::from_millis(TICK_RATE_MS))
         });
     }
 
     thread::spawn(move || loop {
-        match event::read().unwrap() {
-            Event::Key(key) => {
-                tx.send(ThokEvent::Key(key)).unwrap();
-            }
-            Event::Resize(_, _) => {
-                tx.send(ThokEvent::Resize).unwrap();
-            }
-            _ => {}
+        let evt = match event::read().unwrap() {
+            Event::Key(key) => Some(ThokEvent::Key(key)),
+            Event::Resize(_, _) => Some(ThokEvent::Resize),
+            _ => None,
+        };
+
+        if evt.is_some() && tx.send(evt.unwrap()).is_err() {
+            break;
         }
     });
 
     rx
 }
 
-fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
+fn ui<B: Backend>(app: &mut App, f: &mut Frame<B>) {
     f.render_widget(&app.thok, f.size());
 }
